var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { html } from "lit";
import { property, query, state } from "lit/decorators.js";
import { classMap } from "lit/directives/class-map.js";
import { FtLitElement, toFtFormComponent } from "@fluid-topics/ft-wc-utils";
import { FtTypography, FtTypographyBody1 } from "@fluid-topics/ft-typography";
import { FtResizer } from "@fluid-topics/ft-resizer";
import { FtInputLabel } from "@fluid-topics/ft-input-label";
import { FtRipple } from "@fluid-topics/ft-ripple";
import { FtTextAreaCssVariables, styles } from "./ft-text-area.styles";
class FtTextArea extends toFtFormComponent(FtLitElement, "textbox") {
    constructor() {
        super(...arguments);
        this.label = "";
        this.value = "";
        this.helper = "";
        this.outlined = false;
        this.disabled = false;
        this.error = false;
        this.height = 50;
        this.focused = false;
    }
    render() {
        const classes = {
            "ft-text-area": true,
            "ft-text-area--filled": !this.outlined,
            "ft-text-area--outlined": this.outlined,
            "ft-text-area--disabled": this.disabled,
            "ft-text-area--has-value": !!this.value,
            "ft-text-area--no-label": !this.label,
            "ft-text-area--in-error": this.error,
        };
        return html `
            <style>
                .ft-text-area--main-panel {
                    height: calc(4 * ${FtTextAreaCssVariables.verticalSpacing} + ${FtTextAreaCssVariables.labelSize} + ${FtTextAreaCssVariables.fontSize} + ${this.height}px);
                }

                ft-resizer {
                    right: ${this.textarea != null ? this.textarea.offsetWidth - this.textarea.clientWidth : 0}px;
                }
            </style>
            <div class="${classMap(classes)}">
                <div class="ft-text-area--main-panel">
                    <ft-input-label text="${this.label}"
                                    ?disabled=${this.disabled}
                                    ?outlined=${this.outlined}
                                    ?raised=${this.focused || this.value != ""}
                                    ?error=${this.error}
                    ></ft-input-label>
                    <div class="ft-text-area--input-panel"
                         aria-label="${this.label}">
                        ${this.outlined ? null : html `
                            <ft-ripple ?disabled=${this.disabled} activated></ft-ripple>
                        `}
                        <textarea class="ft-typography--body1 ft-text-area--input"
                                  ?disabled=${this.disabled}
                                  .value=${this.value}
                                  @change=${this.updateValueFromInputField}
                                  @keydown=${this.onKeyDown}
                                  @keyup=${this.liveUpdateValueFromInputField}
                                  @focus=${() => this.focused = true}
                                  @focusout=${() => this.focused = false}>
                        </textarea>
                    </div>
                    <ft-resizer @resize=${this.onResize}
                                initialHeight="${this.height}"
                                cursor="ns-resize"></ft-resizer>
                </div>
                ${this.helper ? html `
                    <ft-typography class="ft-text-area--helper-text" variant="caption">
                        ${this.helper}
                    </ft-typography>` : undefined}
            </div>
        `;
    }
    setValue(value, sendEvents = false) {
        this.value = value;
        this.setFormValue(value);
        if (sendEvents) {
            this.dispatchEvent(new CustomEvent("live-change", { detail: this.value }));
            this.dispatchEvent(new CustomEvent("change", { detail: this.value }));
        }
    }
    onKeyDown(e) {
        if (e.key === "Enter" || e.key === " ") {
            e.stopPropagation();
        }
    }
    updateValueFromInputField() {
        this.dispatchEvent(new CustomEvent("change", { detail: this.value }));
    }
    liveUpdateValueFromInputField() {
        var _a;
        this.value = ((_a = this.textarea) === null || _a === void 0 ? void 0 : _a.value) || "";
        this.setFormValue(this.value);
        this.dispatchEvent(new CustomEvent("live-change", { detail: this.value }));
    }
    onResize(event) {
        if (event.detail.height >= 50) {
            this.height = event.detail.height;
        }
    }
}
FtTextArea.elementDefinitions = {
    "ft-input-label": FtInputLabel,
    "ft-resizer": FtResizer,
    "ft-ripple": FtRipple,
    "ft-typography": FtTypography,
};
// language=CSS
FtTextArea.styles = [
    FtTypographyBody1,
    styles
];
__decorate([
    property({ type: String })
], FtTextArea.prototype, "label", void 0);
__decorate([
    property({ reflect: true })
], FtTextArea.prototype, "value", void 0);
__decorate([
    property({ type: String })
], FtTextArea.prototype, "helper", void 0);
__decorate([
    property({ type: Boolean })
], FtTextArea.prototype, "outlined", void 0);
__decorate([
    property({ type: Boolean })
], FtTextArea.prototype, "disabled", void 0);
__decorate([
    property({ type: Boolean })
], FtTextArea.prototype, "error", void 0);
__decorate([
    property({ type: Number, reflect: true })
], FtTextArea.prototype, "height", void 0);
__decorate([
    query("textarea")
], FtTextArea.prototype, "textarea", void 0);
__decorate([
    state()
], FtTextArea.prototype, "focused", void 0);
export { FtTextArea };
