import { css } from "lit";
import { designSystemVariables, FtCssVariableFactory, setVariable } from "@fluid-topics/ft-wc-utils";
import { FtTypographyBody1CssVariables } from "@fluid-topics/ft-typography/build/ft-typography.styles";
import { FtInputLabelCssVariables } from "@fluid-topics/ft-input-label/build/ft-input-label.styles";
import { FtRippleCssVariables } from "@fluid-topics/ft-ripple/build/ft-ripple.styles";
export const FtTextAreaCssVariables = {
    fontSize: FtCssVariableFactory.create("--ft-text-area-font-size", "", "SIZE", "14px"),
    labelSize: FtCssVariableFactory.create("--ft-text-area-label-size", "", "SIZE", "11px"),
    verticalSpacing: FtCssVariableFactory.create("--ft-text-area-vertical-spacing", "", "SIZE", "4px"),
    horizontalSpacing: FtCssVariableFactory.create("--ft-text-area-horizontal-spacing", "", "SIZE", "16px"),
    helperColor: FtCssVariableFactory.extend("--ft-text-area-helper-color", "", designSystemVariables.colorOnSurfaceMedium),
    colorPrimary: FtCssVariableFactory.external(designSystemVariables.colorPrimary, "Design system"),
    colorOnSurface: FtCssVariableFactory.external(designSystemVariables.colorOnSurface, "Design system"),
    colorOnSurfaceDisabled: FtCssVariableFactory.external(designSystemVariables.colorOnSurfaceDisabled, "Design system"),
    borderRadiusS: FtCssVariableFactory.external(designSystemVariables.borderRadiusS, "Design system"),
    colorError: FtCssVariableFactory.external(designSystemVariables.colorError, "Design system"),
};
// language=CSS
export const styles = css `
    *:focus {
        outline: none;
    }

    .ft-text-area {
        display: flex;
        flex-direction: column;
        align-items: stretch;
    }

    ft-input-label {
        ${setVariable(FtInputLabelCssVariables.fontSize, FtTextAreaCssVariables.fontSize)};
        ${setVariable(FtInputLabelCssVariables.raisedFontSize, FtTextAreaCssVariables.labelSize)};
        ${setVariable(FtInputLabelCssVariables.verticalSpacing, FtTextAreaCssVariables.verticalSpacing)};
        ${setVariable(FtInputLabelCssVariables.horizontalSpacing, FtTextAreaCssVariables.horizontalSpacing)};
    }

    .ft-text-area--main-panel {
        position: relative;
        display: flex;
    }

    .ft-text-area--input-panel {
        flex-grow: 1;
        position: relative;
        display: flex;
        align-items: center;
        overflow: hidden;
        gap: 8px;
    }

    .ft-text-area--input-panel ft-ripple {
        ${setVariable(FtRippleCssVariables.opacityContentOnSurfaceHover, "0.08")};
        ${setVariable(FtRippleCssVariables.opacityContentOnSurfacePressed, "0.04")};
    }

    .ft-text-area--input {
        display: block;
        position: relative;
        flex-grow: 1;
        flex-shrink: 1;
        min-width: 0; /* flex sets this to auto an prevents input from shrinking properly */
        align-self: stretch;
        resize: none;

        ${setVariable(FtTypographyBody1CssVariables.fontSize, FtTextAreaCssVariables.fontSize)};
        ${setVariable(FtTypographyBody1CssVariables.lineHeight, FtTextAreaCssVariables.fontSize)};
        color: ${FtTextAreaCssVariables.colorOnSurface};

        padding: 0 ${FtTextAreaCssVariables.horizontalSpacing};
        margin: calc(${FtTextAreaCssVariables.verticalSpacing} + ${FtTextAreaCssVariables.labelSize} / 2) 0 ${FtTextAreaCssVariables.verticalSpacing} 0;

        border: none;
        background: none;

        line-height: 1.4;
    }

    .ft-text-area--disabled .ft-text-area--input {
        color: ${FtTextAreaCssVariables.colorOnSurfaceDisabled};
    }

    .ft-text-area:not(.ft-text-area--disabled) .ft-text-area--input-panel {
        cursor: pointer;
    }

    .ft-text-area:not(.ft-text-area--disabled):focus-within ft-input-label {
        ${setVariable(FtInputLabelCssVariables.borderColor, FtTextAreaCssVariables.colorPrimary)};
        ${setVariable(FtInputLabelCssVariables.textColor, FtTextAreaCssVariables.colorPrimary)};
    }

    .ft-text-area--filled .ft-text-area--input-panel {
        border-radius: ${FtTextAreaCssVariables.borderRadiusS} ${FtTextAreaCssVariables.borderRadiusS} 0 0;
    }

    .ft-text-area--filled:not(.ft-text-area--no-label) .ft-text-area--input {
        align-self: stretch;
        margin-bottom: ${FtTextAreaCssVariables.verticalSpacing};
        margin-top: calc(${FtTextAreaCssVariables.labelSize} + 2 * ${FtTextAreaCssVariables.verticalSpacing});
    }

    .ft-text-area--outlined .ft-text-area--input-panel {
        border-radius: ${FtTextAreaCssVariables.borderRadiusS};
    }

    .ft-text-area--helper-text {
        padding: 0 12px 0 ${FtTextAreaCssVariables.horizontalSpacing};
        color: ${FtTextAreaCssVariables.helperColor};
    }

    .ft-text-area--in-error .ft-text-area--input,
    .ft-text-area--in-error .ft-text-area--helper-text {
        color: ${FtTextAreaCssVariables.colorError};
    }

    ft-resizer {
        position: absolute;
        bottom: 0;
    }
`;
